@@ -14,7 +14,6 @@ from __future__ import absolute_import
 
 import posixpath
 import os
-import six
 import json
 import numpy as np
 import logging
@@ -23,7 +22,7 @@ from collections.abc import (
     Mapping, MutableMapping, KeysView, ValuesView, ItemsView
 )
 from .objectid import GroupID
-from .h5type import Reference, check_dtype
+from .h5type import Reference, check_dtype, special_dtype
 
 numpy_integer_types = (np.int8, np.uint8, np.int16, np.int16, np.int32, np.uint32, np.int64, np.uint64)
 numpy_float_types = (np.float16, np.float32, np.float64)
@@ -59,6 +58,36 @@ def with_phil(func):
     functools.update_wrapper(wrapper, func, ('__name__', '__doc__'))
     return wrapper
 
+def find_item_type(data):
+    """Find the item type of a simple object or collection of objects.
+
+    E.g. [[['a']]] -> str
+
+    The focus is on collections where all items have the same type; we'll return
+    None if that's not the case.
+
+    The aim is to treat numpy arrays of Python objects like normal Python
+    collections, while treating arrays with specific dtypes differently.
+    We're also only interested in array-like collections - lists and tuples,
+    possibly nested - not things like sets or dicts.
+    """
+    if isinstance(data, np.ndarray):
+        if (
+            data.dtype.kind == 'O'
+            and not check_dtype(vlen=data.dtype)
+        ):
+            item_types = {type(e) for e in data.flat}
+        else:
+            return None
+    elif isinstance(data, (list, tuple)):
+        item_types = {find_item_type(e) for e in data}
+    else:
+        return type(data)
+
+    if len(item_types) != 1:
+        return None
+    return item_types.pop()
+
 
 def guess_dtype(data):
     """ Attempt to guess an appropriate dtype for the object, returning None
@@ -66,11 +95,48 @@ def guess_dtype(data):
     constructor to figure out)
     """
 
-    # todo - handle RegionReference, Reference, vlen dtypes
-    if isinstance(data, np.ndarray):
-        return data.dtype
+    # todo - handle RegionReference, Reference
+    item_type = find_item_type(data)
+    if item_type is bytes:
+        return special_dtype(vlen=bytes)
+    if item_type is str:
+        return special_dtype(vlen=str)
+
     return None
 
+def is_float16_dtype(dt):
+    if dt is None:
+        return False
+
+    dt = np.dtype(dt)  # normalize strings -> np.dtype objects
+    return dt.kind == 'f' and dt.itemsize == 2
+
+def array_for_new_object(data, specified_dtype=None):
+    """Prepare an array from data used to create a new dataset or attribute"""
+
+    # We mostly let HDF5 convert data as necessary when it's written.
+    # But if we are going to a float16 datatype, pre-convert in python
+    # to workaround a bug in the conversion.
+    # https://github.com/h5py/h5py/issues/819
+    if is_float16_dtype(specified_dtype):
+        as_dtype = specified_dtype
+    elif not isinstance(data, np.ndarray) and (specified_dtype is not None):
+        # If we need to convert e.g. a list to an array, don't leave numpy
+        # to guess a dtype we already know.
+        as_dtype = specified_dtype
+    else:
+        as_dtype = guess_dtype(data)
+
+    data = np.asarray(data, order="C", dtype=as_dtype)
+
+    # In most cases, this does nothing. But if data was already an array,
+    # and as_dtype is a tagged h5py dtype (e.g. for an object array of strings),
+    # asarray() doesn't replace its dtype object. This gives it the tagged dtype:
+    if as_dtype is not None:
+        data = data.view(dtype=as_dtype)
+
+    return data
+
 
 def _decode(item, encoding="ascii"):
         """decode any byte items to python 3 strings
@@ -149,9 +215,7 @@ def copyToArray(arr, rank, index, data, vlen_base=None):
             copyToArray(arr, rank+1, index, data[i], vlen_base=vlen_base)
         else:
             if vlen_base:
-                if six.PY2 and vlen_base == unicode:
-                    e = unicode(data[i])
-                elif vlen_base == str:
+                if vlen_base == str:
                     e = str(data[i])
                 else:
                     e = np.array(data[i], dtype=vlen_base)
@@ -250,10 +314,10 @@ def isVlen(dt):
     return is_vlen
 
 """
-Get number of byte needed to given element as a bytestream
+Get number of byte needed for given element as a bytestream
 """
 def getElementSize(e, dt):
-    #print("getElementSize - e: {}  dt: {}".format(e, dt))
+    # print(f"getElementSize - e: {e}  dt: {dt} itemsize: {dt.itemsize}")
     if len(dt) > 1:
         count = 0
         for name in dt.names:
@@ -269,7 +333,7 @@ def getElementSize(e, dt):
             if e == 0:
                 count = 4  # non-initialized element
             else:
-                raise ValueError("Unexpected value: {}".format(e))
+                raise ValueError(f"Unexpected value: {e}")
         elif isinstance(e, bytes):
             count = len(e) + 4
         elif isinstance(e, str):
@@ -279,10 +343,7 @@ def getElementSize(e, dt):
             if e.dtype.kind != 'O':
                 count = e.dtype.itemsize * nElements
             else:
-                arr1d = e.reshape((nElements,))
-                count = 0
-                for item in arr1d:
-                    count += getElementSize(item, dt)
+                count = nElements * vlen.itemsize # tbd - special case for strings?
             count += 4  # byte count
         elif isinstance(e, list) or isinstance(e, tuple):
             if not e:
@@ -292,7 +353,7 @@ def getElementSize(e, dt):
                 count = len(e) * vlen.itemsize + 4  # +4 for byte count
         else:
 
-            raise TypeError("unexpected type: {}".format(type(e)))
+            raise TypeError(f"unexpected type: {type(e)}")
     return count
 
 
@@ -300,7 +361,8 @@ def getElementSize(e, dt):
 Get number of bytes needed to store given numpy array as a bytestream
 """
 def getByteArraySize(arr):
-    if not isVlen(arr.dtype):
+    if not isVlen(arr.dtype) and arr.dtype.kind != 'O':
+        print("not isVlen")
         return arr.itemsize * np.prod(arr.shape)
     nElements = int(np.prod(arr.shape))
     # reshape to 1d for easier iteration
@@ -355,7 +417,11 @@ def copyElement(e, dt, buffer, offset, vlen=None):
             offset = copyBuffer(count.tobytes(), buffer, offset)
             offset = copyBuffer(e, buffer, offset)
         elif isinstance(e, str):
-            text = e.encode('utf-8')
+            if vlen == str:
+                encoding = "utf-8"
+            else:
+                encoding = "ascii"
+            text = e.encode(encoding)
             count = np.int32(len(text))
             offset = copyBuffer(count.tobytes(), buffer, offset)
             offset = copyBuffer(text, buffer, offset)
@@ -368,10 +434,15 @@ def copyElement(e, dt, buffer, offset, vlen=None):
                 offset = copyBuffer(e.tobytes(), buffer, offset)
             else:
                 arr1d = e.reshape((nElements,))
-                for item in arr1d:
-                    offset = copyElement(item, dt, buffer, offset)
+                count = np.int32(nElements * vlen.itemsize)
+                offset = copyBuffer(count.tobytes(), buffer, offset)
+                arr = np.asarray(arr1d, dtype=vlen)
+                offset = copyBuffer(arr.tobytes(), buffer, offset)
 
-        elif False and isinstance(e, list) or isinstance(e, tuple):
+                #for item in arr1d:
+                #    offset = copyElement(item, dt, buffer, offset)
+
+        elif isinstance(e, list) or isinstance(e, tuple):
             count = np.int32(len(e) * vlen.itemsize)
             offset = copyBuffer(count.tobytes(), buffer, offset)
             if isinstance(e, np.ndarray):
@@ -409,7 +480,7 @@ def getElementCount(buffer, offset):
 Read element from bytearrray
 """
 def readElement(buffer, offset, arr, index, dt):
-    #print("readElement, offset: {}, index: {} dt: {}".format(offset, index, dt))
+    # print(f"readElement, offset: {offset}, index: {index} dt: {dt}")
 
     if len(dt) > 1:
         e = arr[index]
@@ -440,22 +511,20 @@ def readElement(buffer, offset, arr, index, dt):
         else:
             count = getElementCount(buffer, offset)
             offset += 4
-            if count > 0:
-                e_buffer = buffer[offset:(offset+count)]
-                offset += count
+            if count < 0:
+                raise ValueError("Unexpected variable length data format")
+            e_buffer = buffer[offset:(offset+count)]
+            offset += count
 
-                if vlen is bytes:
-                    arr[index] = bytes(e_buffer)
-                elif vlen is str:
-                    s = e_buffer.decode("utf-8")
-                    arr[index] = s
-                else:
-                    try:
-                        e = np.frombuffer(bytes(e_buffer), dtype=vlen)
-                    except ValueError:
-                        print("ValueError -- e_buffer:", e_buffer, "dtype:", vlen)
-                        raise
-                    arr[index] = e
+            if vlen in (bytes, str):
+                arr[index] = bytes(e_buffer)
+            else:
+                try:
+                    e = np.frombuffer(bytes(e_buffer), dtype=vlen)
+                except ValueError:
+                    print("ValueError -- e_buffer:", e_buffer, "dtype:", vlen)
+                    raise
+                arr[index] = e        
 
     return offset
 
@@ -492,11 +561,12 @@ def bytesToArray(data, dt, shape):
         for index in range(nelements):
             offset = readElement(data, offset, arr, index, dt)
     
-    if shape == () and dt.shape:
-        # special case for scalar array with array sub-type
-        arr = arr.reshape(dt.shape)
-    else:
-        arr = arr.reshape(shape)
+    if shape is not None:
+        if shape == () and dt.shape:
+            # special case for scalar array with array sub-type
+            arr = arr.reshape(dt.shape)
+        else:
+            arr = arr.reshape(shape)
     return arr
 
 
@@ -1120,6 +1190,9 @@ class Empty(object):
         the same as an array with shape (0,).
     """
 
+    shape = None
+    size = None
+
     def __init__(self, dtype):
         self.dtype = np.dtype(dtype)
 
